package de.hft.stuttgart.citydoctor2.healing;

import java.util.HashSet;
import java.util.Set;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hft.stuttgart.citydoctor2.check.HealingID;
import de.hft.stuttgart.citydoctor2.check.HealingMethod;
import de.hft.stuttgart.citydoctor2.check.ModificationListener;
import de.hft.stuttgart.citydoctor2.check.error.NonPlanarPolygonDistancePlaneError;
import de.hft.stuttgart.citydoctor2.datastructure.LinearRing;
import de.hft.stuttgart.citydoctor2.datastructure.Polygon;
import de.hft.stuttgart.citydoctor2.datastructure.Vertex;
import de.hft.stuttgart.citydoctor2.math.Plane;
import de.hft.stuttgart.citydoctor2.math.Vector3d;

public class HealPlanarPolygonError implements HealingMethod {

	private static final Logger logger = LogManager.getLogger(HealPlanarPolygonError.class);

	@Override
	public boolean visit(NonPlanarPolygonDistancePlaneError err, ModificationListener l) {
		logger.debug("Executing Repair for NonPlanarPolygonDistancePlaneError");
		Plane plane = err.getPlane();
		Polygon p = err.getPolygon();
		Set<Vertex> vertices = new HashSet<>();
		// collect all vertices, eliminating duplicates
		collectVertices(p, vertices);
		// project each vertex to the plane and change the coordinates of existing
		// vertices
		for (Vertex v : vertices) {
			Vector3d newPoint = plane.projectPointToPlane(v);
			v.setX(newPoint.getX());
			v.setY(newPoint.getY());
			v.setZ(newPoint.getZ());
		}
		return true;
	}

	private void collectVertices(Polygon p, Set<Vertex> vertices) {
		collectVertices(p.getExteriorRing(), vertices);
		for (LinearRing lr : p.getInnerRings()) {
			collectVertices(lr, vertices);
		}
	}

	private void collectVertices(LinearRing ring, Set<Vertex> vertices) {
		for (Vertex v : ring.getVertices()) {
			vertices.add(v);
		}
	}

	@Override
	public HealPlanarPolygonError createNew() {
		return new HealPlanarPolygonError();
	}

	@Override
	public HealingID getID() {
		return HealingID.P_NON_PLANAR_POLYGON;
	}
}
