package de.hft.stuttgart.citydoctor2.ziploader;

import de.hft.stuttgart.citydoctor2.check.Checker;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import de.hft.stuttgart.citydoctor2.ziploader.utils.ArchivePacker;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.Serial;

import java.io.Serializable;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.List;

import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

public class CityGmlArchive implements Serializable {

    private static final Logger logger = LogManager.getLogger(CityGmlArchive.class);

    @Serial
    private static final long serialVersionUID = 2168389511043362615L;

    private final List<CityGmlZipEntry> entries;
    private final Path archivePath;

    public static CityGmlArchive fromZipFile(String zipFile, ParserConfiguration config) {
        ArrayList<CityGmlZipEntry> archiveEntries = new ArrayList<>();
        ZipFile zip = null;
        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFile));) {
            zip = new ZipFile(zipFile);
            ZipEntry ze;
            while ((ze = zis.getNextEntry()) != null )
            {
                if (ze.isDirectory()) {
                    continue;
                }
                if (ze.getName().endsWith(".gml")) {
                    archiveEntries.add(CityGmlZipEntry.of(ze, zip, config));
                }
            }
        } catch (IOException e) {
            logger.error(e);
        } finally {
            if(zip != null){
                try {
                    zip.close();
                } catch (IOException e) {
                    logger.error(e);
                }
            }
        }
        logger.info("Zip-archive successfully uncompressed, {} CityGml files detected", archiveEntries.size());
        return new CityGmlArchive(archiveEntries, Path.of(zipFile));
    }

    private CityGmlArchive(List<CityGmlZipEntry> entries, Path archivePath) {
        this.entries = entries;
        entries.forEach(e -> e.setArchive(this));
        this.archivePath = archivePath;
    }

    public void exportToZipFile(String path)  {
        ArchivePacker.packArchive(path, this);
    }

    public void checkEntries(){
        for(CityGmlZipEntry entry : entries){
            Checker checker = new Checker(entry.getModel());
            checker.runChecks();
        }
    }

    public Path getArchivePath() {
        return archivePath;
    }

    public List<CityGmlZipEntry> getEntries() {
        return entries;
    }
}
